// ///////////////////////////////////////////////////////////////
// No warranty is expressed or implied.
// MyMoney, MyMoney Jr. (etc.) are registered trademarks of MTH,Inc.
// All other company and product names are trademarks are the property of their
// respective holders.
// Copyright 2006 by MTH,Inc. Printed in USA. All rights reserved.
//
// ////////////////////////////////////////////////////////////////
// FILE :
//
// DESCRIPTION :
//
// OWNER : Tim
//
// ====================================================================
//
// $Header: /cvsroot/MyMoney/src/com/mth/mymoney/components/screenlocker/matrix/MatrixCharacter.java,v 1.1 2007/11/27 06:40:10 tim Exp $
// $Revision: 1.1 $
// $Date: 2007/11/27 06:40:10 $



package com.mth.mymoney.components.screenlocker.matrix;



import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.ImageObserver;
import java.net.URL;

/**
 * 
 * @author
 */
class MatrixCharacter implements ImageObserver {

    // _numOfCharacters should indicate the number of characters (rows) in the
    // image
    private final int _numOfCharacters = 27;
    private int       _charHeight;
    private int       _charWidth;
    private Image     _image;


    /**
     * Creates a new instance of CharcterPainter
     * 
     * we assume the image contains two columns of characters, where the second
     * column conatins the bright versions of the characters used for throws
     * initial character paint.
     */
    MatrixCharacter(MatrixScreenLocker screenSaver) {

        int imageWidth, imageHeight;

        _image = getMatrixImage( screenSaver );

        do {
            imageWidth = _image.getWidth( this );
        }
        while ( imageWidth == -1 );
        do {
            imageHeight = _image.getHeight( this );
        }
        while ( imageHeight == -1 );

        _charWidth = imageWidth / 2;
        _charHeight = imageHeight
                      / _numOfCharacters;
    }

    /**
     * 
     * @param screenSaver
     * @return
     */
    private Image getMatrixImage( MatrixScreenLocker screenSaver ) {

        URL url = MatrixScreenLocker.class.getResource( screenSaver.getFontFileName() );

        Image img = Toolkit.getDefaultToolkit()
                           .getImage( url );

        try {
            MediaTracker tracker = new MediaTracker( screenSaver );
            tracker.addImage( img,
                              0 );
            tracker.waitForID( 0 );
        }
        catch ( Exception e ) {}

        return img;
    }


    // paints a character at the specified position
    // x and y specify the column and row
    public void paintCharacter( int charNum,
                                Graphics2D g2,
                                int col,
                                int row,
                                boolean bright )
    {

        int x, y;
        int _x, _y;

        x = col
            * _charWidth;
        y = row
            * _charHeight;

        _x = bright
                   ? _charWidth
                   : 0;
        _y = charNum
             * _charHeight;

        g2.drawImage( _image,
                      x,
                      y,
                      x
                              + _charWidth,
                      y
                              + _charHeight,
                      _x,
                      _y,
                      _x
                              + _charWidth,
                      _y
                              + _charHeight,
                      this );
    }

    // totally erases the character at the specified position
    public void eraseCharacter( Graphics2D g2,
                                int col,
                                int row )
    {

        g2.clearRect( col
                              * _charWidth,
                      row
                              * _charHeight,
                      _charWidth,
                      _charHeight );
    }

    public int getNumOfCharacters() {

        return _numOfCharacters;
    }

    public int getCharWidth() {

        return _charWidth;
    }

    public int getCharHeight() {

        return _charHeight;
    }

    // need this method to support ImageObserver interface for async awt method
    // calls
    public boolean imageUpdate( Image img,
                                int infoflags,
                                int x,
                                int y,
                                int width,
                                int height )
    {

        return false;
    }

}
