// ///////////////////////////////////////////////////////////////
// No warranty is expressed or implied.
// MyMoney, MyMoney Jr. (etc.) are registered trademarks of MTH,Inc.
// All other company and product names are trademarks are the property of their
// respective holders.
// Copyright 2006 by MTH,Inc. Printed in USA. All rights reserved.
//
// ////////////////////////////////////////////////////////////////
// FILE :
//
// DESCRIPTION :
//
// OWNER : Tim
//
// ====================================================================
//
// $Header:
// /cvsroot/MyMoney/src/com/mth/mymoney/components/screenlocker/matrix/MatrixScreenLocker.java,v
// 1.2 2007/11/27 06:40:10 tim Exp $
// $Revision: 1.3 $
// $Date: 2007/11/28 20:35:22 $




package com.mth.mymoney.components.screenlocker.matrix;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.jidesoft.plaf.LookAndFeelFactory;
import com.mth.mymoney.config.Environment;
import com.mth.mymoney.shared.Constants;
import com.mth.mymoney.shared.i18n.I18n;



/**
 * 
 * Some code from Unix by jamie zawinski
 * 
 * 
 * @author Tim
 * 
 */
@SuppressWarnings("serial")
public class MatrixScreenLocker extends Component implements ImageObserver, Runnable
{

    private Thread          anim;
    private BufferedImage   bufferedImage;
    private MatrixCharacter charPainter;
    private int             numOfColumns;
    private int             numOfRows;
    private int             charHeight;
    private int             charWidth;
    private final int       delay = 40;

    private MatrixCode      lineArray[];
    private Component       screenComponent;


    /**
     * 
     * @param g
     */
    public void init( Component c ) {

        screenComponent = c;

        Dimension d = c.getSize()
                       .getSize();

        charPainter = new MatrixCharacter( this );
        charHeight = charPainter.getCharHeight();
        charWidth = charPainter.getCharWidth();

        numOfColumns = d.width
                       / charWidth;
        numOfRows = d.height
                    / charHeight;
        lineArray = new MatrixCode[numOfColumns];

        for ( int i = 0; i < numOfColumns; i++ ) {

            lineArray[i] = new MatrixCode( i,
                                           this );
        }

    }


    /**
     * 
     * @return
     */
    public MatrixCharacter getCharacterPainter() {

        return charPainter;
    }

    /**
     * 
     * @return
     */
    public int getNumOfColumns() {

        return numOfColumns;
    }

    /**
     * 
     * @return
     */
    public int getNumOfRows() {

        return numOfRows;
    }

    /**
     * 
     * @return
     */
    public String getFontFileName() {

        return "small_matrix_font.png";
    }

    /**
     * 
     * @param w
     * @param h
     * @param g2
     */
    public void updateBuffer( int w,
                             int h,
                             Graphics2D g2 )
    {

        for ( int i = 0; i < numOfColumns; i++ ) {
            lineArray[i].paint( g2 );
        }
    }


    /**
     * 
     * @param w
     * @param h
     * @return
     */
    public Graphics2D createGraphics2D( int w,
                                       int h )
    {

        Graphics2D g2 = null;

        if ( bufferedImage == null
             || bufferedImage.getWidth() != w
             || bufferedImage.getHeight() != h )
        {
            bufferedImage = new BufferedImage( w,
                                               h,
                                               BufferedImage.TYPE_INT_ARGB );
        }

        g2 = bufferedImage.createGraphics();
        g2.setRenderingHint( RenderingHints.KEY_RENDERING,
                             RenderingHints.VALUE_RENDER_QUALITY );

        // Set best alpha interpolation quality
        //
        g2.setRenderingHint( RenderingHints.KEY_ALPHA_INTERPOLATION,
                             RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY );

        return g2;
    }


    /**
     * 
     */
    public void paint( Graphics g ) {

        if ( g == null )
            return;
        
        Dimension d = screenComponent.getSize();

        Graphics2D g2 = createGraphics2D( d.width,
                                          d.height );
        // draw to buffer
        updateBuffer( d.width,
                      d.height,
                      g2 );

        g2.dispose();
        // copy buffer image to screen


        g.drawImage( bufferedImage,
                     0,
                     0,
                     this );
    }

    /**
     * 
     */
    public boolean imageUpdate( Image img,
                               int infoflags,
                               int x,
                               int y,
                               int width,
                               int height )
    {

        return false;
    }

    /**
     * 
     */
    public void start() {

        anim = new Thread( this );
        anim.start();
    }

    /**
     * 
     * @param c
     */
    public void startInWindow( Component c ) {

        init( c );
        start();
    }

    /**
     * 
     */
    public synchronized void stop() {

        if ( anim != null ) {

            anim = null;

            setBackground( screenComponent.getBackground() );
            screenComponent.invalidate();
            screenComponent.repaint();
            notify();
            notifyAll();
        }
    }

    /**
     * 
     */
    public synchronized void run() {

        while ( Thread.currentThread() == anim ) {

            if ( screenComponent != null )
                paint( screenComponent.getGraphics() );

            try {
                wait( delay );
            }
            catch ( InterruptedException e ) {
                return;
            }
        }
    }


    /***************************************************************************
     * 
     * 
     * @param argv
     */
    public static void main( String argv[] ) {

        I18n.maybeLoad();
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        LookAndFeelFactory.installJideExtension( LookAndFeelFactory.OFFICE2003_STYLE );


        Environment.setTransientProperty( Constants.DEBUG_FLAG,
                                          true );


        final MatrixScreenLocker demo = new MatrixScreenLocker();


        final JFrame frame = new JFrame( "Matrix Code Simulation" );

        final JPanel panel = new JPanel();
        final JButton attachedButton = new JButton( "Click Me" );

        frame.addMouseMotionListener( new MouseMotionListener() {

            public void mouseDragged( MouseEvent e ) {

                demo.stop();

            }

            public void mouseMoved( MouseEvent e ) {

                demo.stop();

            }
        } );


        AbstractAction action = new AbstractAction( "Test of a frame locker" ) {

            public void actionPerformed( ActionEvent e ) {

                demo.startInWindow( frame );
            }
        };
        attachedButton.setAction( action );

        panel.add( attachedButton,
                   BorderLayout.CENTER );



        frame.addWindowListener( new WindowAdapter() {

            public void windowClosing( WindowEvent e ) {

                System.exit( 0 );
            }
        } );

        frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

        frame.setLocationRelativeTo( null );

        frame.add( panel );
        frame.pack();
        frame.setVisible( true );

    }


}
